﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyFileImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyFileImplement.Implements
{
    public class ProductStorage : IProductStorage
    {
        private readonly DataFileSingleton source;

        public ProductStorage()
        {
            source = DataFileSingleton.GetInstance();
        }

        public ProductViewModel? Delete(ProductBindingModel model)
        {
            var element = source.Products.FirstOrDefault(x => x.Id == model.Id);

            if (element != null)
            {
                source.Products.Remove(element);
                source.SaveProducts();

                return element.GetViewModel;
            }

            return null;
        }

        public ProductViewModel? GetElement(ProductSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ProductName) && !model.Id.HasValue)
            {
                return null;
            }

            return source.Products.FirstOrDefault(x => (!string.IsNullOrEmpty(model.ProductName) && x.ProductName == model.ProductName) || (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }

        public List<ProductViewModel> GetFilteredList(ProductSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ProductName))
            {
                return new();
            }

            return source.Products.Where(x => x.ProductName.Contains(model.ProductName)).Select(x => x.GetViewModel).ToList();
        }

        public List<ProductViewModel> GetFullList()
        {
            return source.Products.Select(x => x.GetViewModel).ToList();
        }

        public ProductViewModel? Insert(ProductBindingModel model)
        {
            model.Id = source.Products.Count > 0 ? source.Products.Max(x => x.Id) + 1 : 1;
            var newProduct = Product.Create(model);

            if (newProduct == null)
            {
                return null;
            }

            source.Products.Add(newProduct);
            source.SaveProducts();

            return newProduct.GetViewModel;
        }

        public ProductViewModel? Update(ProductBindingModel model)
        {
            var product = source.Products.FirstOrDefault(x => x.Id == model.Id);

            if (product == null)
            {
                return null;
            }

            product.Update(model);
            source.SaveProducts();

            return product.GetViewModel;
        }
    }
}
